Class: Command::GithubFlowReadiness

Inherits:
Base
  • Object
show all
Defined in:
lib/command/github_flow_readiness.rb

Constant Summary collapse

NAME =
"github-flow-readiness"
DESCRIPTION =
"Checks whether the current repo is ready for the Control Plane GitHub flow rollout"
LONG_DESCRIPTION =
<<~DESC
  Checks the current repository for common rollout blockers before adding the Control Plane GitHub flow:
  - Rails runtime scaffold present
  - modern Ruby and Bundler toolchain
  - installable exact-pinned direct gem and npm package versions
  - production Dockerfile presence and SQLite production hints
DESC
EXAMPLES =
<<~EX
  ```sh
  # Checks the current repo for common rollout blockers
  cpflow github-flow-readiness
  ```
EX
WITH_INFO_HEADER =
false
VALIDATIONS =
[].freeze
REQUIRES_STARTUP_CHECKS =
false

Constants inherited from Base

Base::ACCEPTS_EXTRA_OPTIONS, Base::ALL_VALIDATIONS, Base::DEFAULT_ARGS, Base::HIDE, Base::OPTIONS, Base::REQUIRES_ARGS, Base::SUBCOMMAND_NAME, Base::USAGE, Base::VALIDATIONS_WITHOUT_ADDITIONAL_OPTIONS, Base::VALIDATIONS_WITH_ADDITIONAL_OPTIONS

Instance Attribute Summary

Attributes inherited from Base

#config

Instance Method Summary collapse

Methods inherited from Base

add_app_identity_option, all_commands, all_options, all_options_by_key_name, app_option, #args_join, commit_option, common_options, #cp, cpu_option, detached_option, dir_option, docker_context_option, domain_option, #ensure_docker_running!, entrypoint_option, image_option, #initialize, interactive_option, location_option, log_method_option, logs_limit_option, logs_since_option, memory_option, org_option, #progress, replica_option, #run_command_in_latest_image, #run_cpflow_command, run_release_phase_option, skip_confirm_option, skip_post_creation_hook_option, skip_pre_deletion_hook_option, skip_secret_access_binding_option, skip_secrets_setup_option, staging_branch_option, #step, #step_finish, terminal_size_option, trace_option, upstream_token_option, use_local_token_option, validations_option, verbose_option, version_option, wait_option, #with_retry, workload_option

Methods included from Helpers

normalize_command_name, normalize_option_name, random_four_digits, strip_str_and_validate

Constructor Details

This class inherits a constructor from Command::Base

Instance Method Details

#callObject



24
25
26
27
28
29
30
31
32
33
34
35
# File 'lib/command/github_flow_readiness.rb', line 24

def call
  service = GithubFlowReadinessService.new

  service.results.each do |result|
    Shell.info("[#{result.status.to_s.upcase}] #{result.message}")
  end

  Shell.info("")
  Shell.info(service.summary)

  exit(ExitCode::ERROR_DEFAULT) if service.blockers?
end