Class: Command::DeployImage
Constant Summary collapse
- NAME =
"deploy-image"- OPTIONS =
[ app_option(required: true), run_release_phase_option, use_digest_image_ref_option ].freeze
- DESCRIPTION =
"Deploys the latest image to app workloads, and runs a release script (optional)"- LONG_DESCRIPTION =
<<~DESC - Deploys the latest image to app workloads - Runs a release script before deploying if `release_script` is specified in the `.controlplane/controlplane.yml` file and `--run-release-phase` is provided - The release script is run in the context of `cpflow run` with the latest image - If the release script exits with a non-zero code, the command will stop executing and also exit with a non-zero code - If `use_digest_image_ref` is `true` in the `.controlplane/controlplane.yml` file or `--use-digest-image-ref` option is provided, deployed image's reference will include its digest - Repairs missing `shared_secret_grants` policy bindings before running a release phase or updating workloads DESC
Constants inherited from Base
Base::ACCEPTS_EXTRA_OPTIONS, Base::ALL_VALIDATIONS, Base::DEFAULT_ARGS, Base::EXAMPLES, Base::HIDE, Base::REQUIRES_ARGS, Base::REQUIRES_STARTUP_CHECKS, Base::SUBCOMMAND_NAME, Base::USAGE, Base::VALIDATIONS, Base::VALIDATIONS_WITHOUT_ADDITIONAL_OPTIONS, Base::VALIDATIONS_WITH_ADDITIONAL_OPTIONS, Base::WITH_INFO_HEADER
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
Methods inherited from Base
add_app_identity_option, all_commands, all_options, all_options_by_key_name, app_option, #args_join, #bind_shared_secret_policy_grant, #bind_shared_secret_policy_grants, commit_option, common_options, #cp, cpu_option, detached_option, dir_option, docker_context_option, domain_option, #ensure_docker_running!, #ensure_shared_secret_policy_targets_secret!, entrypoint_option, force_option, #identity_bound_to_policy_with_reveal?, #identity_policy_permissions, image_option, #initialize, interactive_option, location_option, log_method_option, logs_limit_option, logs_since_option, memory_option, org_option, #progress, replica_option, #resolve_shared_secret_policy_grant, #resolve_shared_secret_policy_grants, #run_command_in_latest_image, #run_cpflow_command, run_release_phase_option, #shared_secret_policy_missing_message, #shared_secret_policy_target_links, #shared_secret_policy_targets_secret?, skip_confirm_option, skip_post_creation_hook_option, skip_pre_deletion_hook_option, skip_secret_access_binding_option, skip_secrets_setup_option, staging_branch_option, #step, #step_finish, terminal_size_option, trace_option, upstream_token_option, use_digest_image_ref_option, use_local_token_option, validations_option, verbose_option, version_option, wait_option, #with_retry, workload_option
Methods included from Helpers
normalize_command_name, normalize_option_name, random_four_digits, strip_str_and_validate
Constructor Details
This class inherits a constructor from Command::Base
Instance Method Details
#call ⇒ Object
23 24 25 26 27 28 29 30 31 32 |
# File 'lib/command/deploy_image.rb', line 23 def call release_script = release_script_to_run image = resolve_image_to_deploy shared_secret_policy_grant_pairs = resolve_shared_secret_policy_grants workload_data_by_name = app_workload_data bind_shared_secret_policy_grants(shared_secret_policy_grant_pairs) run_release_script(release_script) if release_script deploy_image_to_workloads(image, workload_data_by_name) end |